source GLIDEUI.tcl

proc SetStereoCameraEyeOffset { tcltag name value } {
    UpdateStereoCamera $tcltag $name
}

proc UpdateStereoCamera { tcltag name } {    
    puts "START: Stereo Camera"

#    if { $tcltag == {} } {
#	set varDYDX "SLF_DYDX"
#    } else {
#	set varDYDX "SLF_[subst $tcltag]_DYDX"
#    }
#    global $varDYDX

    set varProjection "[subst $name]_projection"
    global $varProjection
    set varIllusion "[subst $name]_illusion"
    global $varIllusion
    set varEyeOffset "[subst $name]_eyeOffset"
    global $varEyeOffset
    set varViewingDistance "[subst $name]_viewingDistance"
    global $varViewingDistance
    set varMetersPerPixel "[subst $name]_metersPerPixel"
    global $varMetersPerPixel

    set varVPWidth "[subst $name]_vpWidth"
    global $varVPWidth
    set varVPHeight "[subst $name]_vpHeight"
    global $varVPHeight

    # Left Frustum Parameters
    set varLeftFrustumLeft "[subst $name]_left_canLeft"
    global $varLeftFrustumLeft
    set varLeftFrustumRight "[subst $name]_left_canRight"
    global $varLeftFrustumRight
    set varLeftFrustumBottom "[subst $name]_left_canBottom"
    global $varLeftFrustumBottom
    set varLeftFrustumTop "[subst $name]_left_canTop"
    global $varLeftFrustumTop

    # Right Frustum Parameters
    set varRightFrustumLeft "[subst $name]_right_canLeft"
    global $varRightFrustumLeft
    set varRightFrustumRight "[subst $name]_right_canRight"
    global $varRightFrustumRight
    set varRightFrustumBottom "[subst $name]_right_canBottom"
    global $varRightFrustumBottom
    set varRightFrustumTop "[subst $name]_right_canTop"
    global $varRightFrustumTop

    # These 3 variables need to be discovered
    set viewerWidth [expr [subst $$varVPWidth]*[subst $$varMetersPerPixel]]
    set viewerHeight [expr [subst $$varVPHeight]*[subst $$varMetersPerPixel]]
    set viewerVPOffset [expr $viewerWidth / 2.0]

    set vpWidth [expr $viewerWidth / [subst $$varViewerDistance]]
    set vpHeight [expr $viewerHeight / [subst $$varViewerDistance]]
    set vpOffset [expr ($viewerVPOffset + [subst $$varEyeOffset]) / \
                       [subst $$varViewerDistance]]

    set $varLeftFrustumLeft [expr $vpOffset - ($vpWidth / 2.0)]
    set $varLeftFrustumRight [expr $vpOffset + ($vpWidth / 2.0)]
    set $varLeftFrustumBottom [expr  -($vpHeight / 2.0)]
    set $varLeftFrustumTop [expr ($vpHeight / 2.0)]

    set $varRightFrustumLeft [expr -$vpOffset - ($vpWidth / 2.0)]
    set $varRightFrustumRight [expr -$vpOffset + ($vpWidth / 2.0)]
    set $varRightFrustumBottom [expr  -($vpHeight / 2.0)]
    set $varRightFrustumTop [expr ($vpHeight / 2.0)]

    puts "END: Stereo Camera"
}

proc CreateStereoCameraUI { tcltag name } {
    set root ".glf_[subst $name]"

    toplevel $root

    ##################################################
    # Projection

    set projectionFrame [CreateProjectionFrame $name $root]

    ##################################################
    # Eye Offset

    set eyeOffsetScale [CreateScaleCmd $name $root eyeOffset "Eye Offset (m)" \
	    0.03 0.000 0.100 0.001 1 horizontal \
	    "SetStereoCameraEyeOffset $tcltag $name"]

    ##################################################
    # Meters Per Pixel

#    set eyeOffsetScale [CreateScale $name $root eyeOffset "Eye Offset (m)" \
#	    0.03 0.000 0.100 0.001 1 horizontal]

    ##################################################
    # Clipping

    set backFrontFrame [CreateDoubleScale $name $root backFront back front \
	    Back Front -10 -2 -0.01 -0.01 0.01 1 horizontal]

    pack $projectionFrame $eyeOffsetScale $backFrontFrame -side top -fill x
}

CreateStereoCameraUI STEREO camStereo



