source GLIDEUI.tcl

proc InitStereoCameraVariables { name } {    
    # Window Variables
    global [subst $name]_win_width
    set [subst $name]_win_width 1
    global [subst $name]_win_height
    set [subst $name]_win_height 1
    
    # VP variables
    global [subst $name]_vp_center_x
    set [subst $name]_vp_center_x 0.5
    global [subst $name]_vp_center_y
    set [subst $name]_vp_center_y 0.5
    
    global [subst $name]_vp_width
    set [subst $name]_vp_width 0.9 
    global [subst $name]_vp_height
    set [subst $name]_vp_height 0.9 
    
    # For cross eyed illusion this should be negative
    global [subst $name]_vp_offset
    set [subst $name]_vp_offset [expr -$[subst $name]_vp_width/4.0] 
    
    # Subviewport size ratio
    global [subst $name]_percent_width
    set [subst $name]_percent_width 0.45
    #set [subst $name]_percent_width 0.5
    global [subst $name]_percent_height
    set [subst $name]_percent_height 0.9
    #set [subst $name]_percent_height 1.0

    # Frustum Stuff
    global [subst $name]_mono
    set [subst $name]_mono 0
    global [subst $name]_illusion
    set [subst $name]_illusion 0
    global [subst $name]_projection
    set [subst $name]_projection 1
    
    global [subst $name]_offset
    set [subst $name]_offset 0.0
    
    global [subst $name]_width
    set [subst $name]_width 1.0
    global [subst $name]_height
    set [subst $name]_height 1.0 
    
    global [subst $name]_back
    set [subst $name]_back -10.0
    global [subst $name]_front
    set [subst $name]_front -0.01
    
    # Placing the camera in the model
    global [subst $name]_vrc_angle
    set [subst $name]_vrc_angle 0.0
    global [subst $name]_vrc_offset
    set [subst $name]_vrc_offset 0.0

    # Real World Viewing Parameters
    global [subst $name]_real_per_pixel # meters per pixel
    set [subst $name]_real_per_pixel 0.00028 
    global [subst $name]_real_distance # Viewing Distance
    set [subst $name]_real_distance 0.57
    global [subst $name]_real_offset # Eye Offset
    set [subst $name]_real_offset 0.0325
}

proc UpdateStereoCameraVariables { name } {    
    ##################################################
    # READ ONLY GLOBAL VARIABLES
    set varWinWidth [subst $name]_win_width
    set varWinHeight [subst $name]_win_height
    global $varWinWidth
    global $varWinHeight

    set varVPWidth [subst $name]_vp_width
    set varVPHeight [subst $name]_vp_height
    global $varVPWidth
    global $varVPHeight

    set varVPOffset [subst $name]_vp_offset
    global $varVPOffset

    set varRealPerPixel [subst $name]_real_per_pixel
    global $varRealPerPixel
    set varViewingDistance [subst $name]_real_distance
    global $varViewingDistance
    set varEyeOffset [subst $name]_real_offset
    global $varEyeOffset

    set varMono [subst $name]_mono
    global $varMono 

    ##################################################
    # READ / WRITE GLOBAL VARIABLES
    global [subst $name]_offset
    
    global [subst $name]_width
    global [subst $name]_height

    ##################################################
    # CONSTRAINT CALCULATIONS

    set width [expr [subst $$varWinWidth] * [subst $$varVPWidth]*[subst $$varRealPerPixel]]
    set height [expr [subst $$varWinHeight] * [subst $$varVPHeight]*[subst $$varRealPerPixel]]

    set fScale [expr 1.0 / [subst $$varViewingDistance]]

    if { [subst $$varMono] == 1 } {
	set [subst $name]_offset 0.0    
    } else {
	# Stereo
	set vpOffset [expr [subst $$varWinWidth]*[subst $$varVPOffset]*[subst $$varRealPerPixel]]
	set [subst $name]_offset [expr ([subst $$varEyeOffset] - $vpOffset)/[subst $$varViewingDistance]]
    }

    set [subst $name]_width [expr $width * $fScale]
    set [subst $name]_height [expr $height * $fScale]

    # Place the cameras
    global [subst $name]_vrc_angle
    global [subst $name]_vrc_offset
    set [subst $name]_vrc_angle 0.0
    set [subst $name]_vrc_offset [subst $[subst $name]_offset]
    

    ##################################################
    # Debugging
    # 

    puts "[subst $name]_width = [subst $[subst $name]_width]"
    puts "[subst $name]_height = [subst $[subst $name]_height]"
    puts "[subst $name]_offset = [subst $[subst $name]_offset]"
    puts "[subst $name]_vrc_angle = [subst $[subst $name]_vrc_angle]"
    puts "[subst $name]_vrc_offset = [subst $[subst $name]_vrc_offset]"

    global [subst $name]_percent_width
    global [subst $name]_percent_height
    global [subst $name]_back
    global [subst $name]_front

    puts "Right: frustum ( [expr -[subst $[subst $name]_offset] - ([subst $[subst $name]_width]* \
             [subst $[subst $name]_percent_width]/2.0)] \
            [expr -([subst $[subst $name]_height]*[subst $[subst $name]_percent_height])/2.0] \
            [expr [subst $[subst $name]_back]] ) \
          ( [expr -[subst $[subst $name]_offset] + ([subst $[subst $name]_width]* \
             [subst $[subst $name]_percent_width]/2.0)] \
            [expr ([subst $[subst $name]_height]*[subst $[subst $name]_percent_height])/2.0] \
            [expr [subst $[subst $name]_front]] )"
}


