## Constants

## Notes:
## PATHSTEPS = # of steps it takes to go around the path
## HALFPATH = # of steps for 1/2 the path.  to make it look less ugly
## QTRTURN = # of steps for 1/4 of the way around 1 turn (ie 1/8 of PATHSTEPS)
## PATHRAD = Radius of the path circles
## MAXY = highest point of the path (in the middle, at times 0 & PATHSTEPS
## CURVEBIAS = correction for nonlinear descent down the center ramps
## ASCENDBIAS = bias to initiate upward angle earlier
## YFLATTEN = correction for plateau at the top of the center ramps.
##            (# of path steps on each circle that are level (not sloped))

#set PATHSTEPS 799
#set HALFPATH 399.0
#set QTRTURN 100.0
set PATHSTEPS 1199
set HALFPATH 599.0
set QTRTURN 150.0
set PATHRAD 3
set MAXY 1.1
set CURVEBIAS .19
set ASCENDBIAS 5
set YFLATTEN 29

source SLFCONSTS.tcl

proc CreateLegTestUI { name } {
    set root ".glf_[subst $name]"

    toplevel $root

    set testAng1 [CreateScale $name $root testAng1 "testAng1" \
	    0.0 -90.0 90.0 1 1 horizontal]

    set testAng2 [CreateScale $name $root testAng2 "testAng2" \
	    0.0 -90.0 90.0 1 1 horizontal]

    set testAng3 [CreateScale $name $root testAng3 "testAng3" \
	    0.0 -90.0 90.0 1 1 horizontal]

    set testAng4 [CreateScale $name $root testAng4 "testAng4" \
	    0.0 -90.0 90.0 1 1 horizontal]

    pack $testAng1 $testAng2 $testAng3 $testAng4 -side top -fill x
}


proc CreateCamUI { name } {
    set root ".glf_[subst $name]"

    toplevel $root

    set zoomIn [CreateScale $name $root zoomIn "Zoom!" \
	    1 0.1 3.0 .05 1 horizontal]

    ## Camera view type
    set camTL {}
    lappend camTL "\"Follow Cam\" 1"
    lappend camTL "\"Chase Cam\" 2"
    lappend camTL "\"Scorpi-Eye(tm) Cam\" 3"
    lappend camTL "\"Trolly Cam\" 4"
    lappend camTL "\"Chopper Cam\" 5"
    lappend camTL "\"CrystalBall Cam\" 6"
    
    set camTogl [CreateRadioButtonFrame $name $root camTogl \
	    "Camera Toggle" 1 $camTL]

    pack $zoomIn $camTogl -side top -fill x

}

proc CreateSceneUI { name } {
    set root ".glf_[subst $name]"

    toplevel $root

    global SLF_OFF
    global SLF_FULL

    ## TimeOfDay
    set dayTL {}
    lappend dayTL "Day 1"
    lappend dayTL "Night 0"

    set dayTogl [CreateRadioButtonFrame $name $root dayTogl \
	    "Day/Night Toggle" 1 $dayTL]


    ## turn on/off the Sky
    set skyTL {}
    lappend skyTL "On $SLF_FULL"
    lappend skyTL "Off $SLF_OFF"

    set skyTogl [CreateRadioButtonFrame $name $root skyTogl \
	    "Sky Toggle\n(Looks bad w/out\nTextures on)" $SLF_FULL $skyTL]

    set bugTL {}
    lappend bugTL "On $SLF_FULL"
    lappend bugTL "Off $SLF_OFF"

    set bugTogl [CreateRadioButtonFrame $name $root bugTogl \
	    "Bug Toggle" $SLF_FULL $bugTL]

    set pathTL {}
    lappend pathTL "On $SLF_FULL"
    lappend pathTL "Off $SLF_OFF"

    set pathTogl [CreateRadioButtonFrame $name $root pathTogl \
	    "Path Toggle" $SLF_FULL $pathTL]

    set cactiTL {}
    lappend cactiTL "On $SLF_FULL"
    lappend cactiTL "Off $SLF_OFF"

    set cactiTogl [CreateRadioButtonFrame $name $root cactiTogl \
	    "Cactus Toggle" $SLF_FULL $cactiTL]

    set pondTL {}
    lappend pondTL "On $SLF_FULL"
    lappend pondTL "Off $SLF_OFF"

    set pondTogl [CreateRadioButtonFrame $name $root pondTogl \
	    "Pond Toggle" $SLF_FULL $pondTL]

    set mtnTL {}
    lappend mtnTL "On $SLF_FULL"
    lappend mtnTL "Off $SLF_OFF"

    set mtnTogl [CreateRadioButtonFrame $name $root mtnTogl \
	    "Mountain Toggle" $SLF_FULL $mtnTL]

    pack $dayTogl $skyTogl $bugTogl $pathTogl $cactiTogl $pondTogl $mtnTogl -side top -fill x
}

proc CreateScorpUI { name } {
    set root ".glf_[subst $name]"

    toplevel $root

    ## turn on/off walking
    set walkTL {}
    lappend walkTL "Go 1"
    lappend walkTL "Stop 0"
    
    set walkTogl [CreateRadioButtonFrame $name $root walkTogl \
	    "Walk Toggle" 1 $walkTL]

    ## walking direction
    set walkSTL {}
    lappend walkSTL "Normal 0"
    lappend walkSTL "Backwards 1"
    lappend walkSTL "\"MoonWalk (MJstyle!)\" 2"
    
    set walkStyl [CreateRadioButtonFrame $name $root walkStyl \
	    "Walking Style" 0 $walkSTL]

    ## time multiplier

    set walkSpeed [CreateScale $name $root walkSpeed "WalkingSpeed * x" \
	    1 1 10 1 1 horizontal]

    ### Eye Direction

    set eyePointY [CreateScale $name $root eyePointY "Eye Direct(L-R)" \
	    0.0 -180.0 180.0 1 1 horizontal]

    set eyePointX [CreateScale $name $root eyePointX "Eye Direct(U-D)" \
	    9.0 -50.0 25.0 1 1 horizontal]

    set pincerAng [CreateScale $name $root pincerAng "Pincers-\nClose<- ->Open" \
	    0.0 -10.0 20.0 1 1 horizontal]

    pack $walkTogl $walkStyl $walkSpeed $eyePointY $eyePointX $pincerAng -side top -fill x
}


proc initglobs {} {

    global ScorpX
    global ScorpY
    global ScorpZ

    set ScorpX 0
    set ScorpY 0
    set ScorpZ 0

    global ScorpAng
    global SlopeAng
    global PincerAng
    
    global WalkTime
    global PathTime

    set WalkTime 0
    set PathTime 0
    set SlopeAng 0
    set PincerAng 0
}

proc updateTime { mult toggle style } {
    global WalkTime
    global PathTime
    
    global PATHSTEPS

    if {$toggle == 0} {
	return
    }
    
    if {$style == 0 || $style == 2} { 
	#normal/moonwalk
	if {$WalkTime >= 20} {
	    set WalkTime [expr 0]
	    
	} else {
	    set WalkTime [expr $WalkTime + 1 * $mult]
	}
    } else { 
	#backwards
	if {$WalkTime <= 0} {
	    set WalkTime [expr 20]
	    
	} else {
	    set WalkTime [expr $WalkTime - 1 * $mult]
	}
    }
    
    if {$style == 0} { 
	#normal
	if {$PathTime >= ($PATHSTEPS - 1) } {
	    set PathTime [expr 0]
	    
	} else {
	    set PathTime [expr $PathTime + 1 * $mult]
	}
    } else { 
	# backwards/moonwalk
	if {$PathTime <= 0 } {
	    set PathTime [expr $PATHSTEPS - 1]
	    
	} else {
	    set PathTime [expr $PathTime - 1 * $mult]
	}
    }
    
}

proc updatePos { style } {
    global PathTime

    global ScorpX
    global ScorpY
    global ScorpZ

    global ScorpAng
    global SlopeAng
    global PincerAng

    global PATHSTEPS
    global HALFPATH
    global QTRTURN
    global PATHRAD
    global MAXY
    global CURVEBIAS
    global ASCENDBIAS
    global YFLATTEN

    global SLF_PI

    set oldX [expr $ScorpX]
    set oldY [expr $ScorpY]
    set oldZ [expr $ScorpZ]

    set SlopeAng 0
    set PincerAng 0

    ## CIRCLE 1 (start high)
    if {$PathTime <= $HALFPATH} {
	set ScorpX [expr cos( $PathTime / $HALFPATH * 2 * $SLF_PI ) \
		* $PATHRAD ]
	set ScorpZ [expr sin( $PathTime / $HALFPATH * 2 * $SLF_PI ) \
		* $PATHRAD ]
	
	## check for plateau, if not on one...

## tried to bias to delay slope... but it "jerks" really bad at frame 34 for
## absolutely no reason, right before the actual frame is displayed. AH!
##	if {$PathTime > $YFLATTEN + $ASCENDBIAS}
if {$PathTime > $YFLATTEN } {
	    if {$PathTime < $QTRTURN} {
		set ScorpY [expr $MAXY * ($QTRTURN - $PathTime) / $QTRTURN \
			+ $CURVEBIAS \
			* sin(($PathTime - $YFLATTEN) \
			/ ($QTRTURN - $YFLATTEN) * $SLF_PI) ]
		
		set dX [expr $ScorpX - $oldX]
		set dY [expr $ScorpY - $oldY]

		if {$dY != 0} {
		    if {$style == 0} {
			set temp [expr atan(($ScorpY - $oldY) \
				/ (sqrt ( $dX * $dX + $dY * $dY ))) ]
			set SlopeAng [RadToDeg $temp]
		    } else {
			set temp [expr -atan(($ScorpY - $oldY) \
				/ (sqrt ( $dX * $dX + $dY * $dY ))) ]
			set SlopeAng [RadToDeg $temp]
		    }
		}
		
	    } else {
		set ScorpY 0
	    }
	} else {
	    ## otherwise, chop it there.
	    set ScorpY [expr $MAXY * ($QTRTURN - $YFLATTEN) / $QTRTURN ]
	}
	
        set ScorpAng [expr ($PathTime / $HALFPATH * -360)]
	
    } else {
	## CIRCLE 2 (start low)
	set ScorpX [expr -cos( ($PathTime - $HALFPATH - 1) \
		/ $HALFPATH * 2 * $SLF_PI ) \
		* $PATHRAD + $PATHRAD * 2 ]
	set ScorpZ [expr sin( ($PathTime - $HALFPATH - 1) \
		/ $HALFPATH \* 2 * $SLF_PI ) * $PATHRAD ]
	
	
	## Check for plateau. if not...
	if {$PathTime < ($PATHSTEPS - $YFLATTEN)} {
	    

	    if {$PathTime > (7 * $QTRTURN - $ASCENDBIAS) } {
		set ScorpY [expr $MAXY * ($PathTime - \
			(7 * $QTRTURN - $ASCENDBIAS)) \
			/ ($QTRTURN + $ASCENDBIAS) \
			+ $CURVEBIAS \
			* sin(($PathTime - (7 * $QTRTURN - $ASCENDBIAS)) \
			/ ($QTRTURN - $YFLATTEN + $ASCENDBIAS) * $SLF_PI) ]
		
		set dX [expr $ScorpX - $oldX]
		set dY [expr $ScorpY - $oldY]
		
		if {$dY != 0} {
		    if {$style == 0 } {
			set temp [expr atan(($ScorpY - $oldY) \
				/ (sqrt ( $dX * $dX + $dY * $dY ))) ]
			set SlopeAng [RadToDeg $temp]
		    } else {
			set temp [expr -atan(($ScorpY - $oldY) \
				/ (sqrt ( $dX * $dX + $dY * $dY ))) ]
			set SlopeAng [RadToDeg $temp]
		    }

		    set PincerAng -18
		}
		
	    } else {
		set ScorpY 0
	    }
	} else {
	    ## Chop Y at plateau
	    set ScorpY [expr $MAXY * ($QTRTURN - $YFLATTEN) / $QTRTURN ]
	}
	
        set ScorpAng [expr ( ($PathTime - $HALFPATH - 1) \
		/ $HALFPATH * 360)]
    }
    
}


proc amIOn { camnum select } {
    if { $camnum == $select } {
	return [expr 3] 
    } else {
	return [expr 0]
    }
}

proc UpperLegAng { offset } {
    global WalkTime
    global SLF_PI

    set pos [expr ($WalkTime + $offset) % 21]

## -27 to 27 deg
    return [expr cos( ($pos) / 20.0 * 2 * $SLF_PI ) * 27]
}

proc LowerLegAng { offset } {
    global WalkTime
    global SLF_PI

    set pos [expr ($WalkTime + $offset + 2) % 21]

    if { $pos >= 10 } {
	## 0 to 40 deg
	return [expr ( sin ( ($pos - 10) / 10.0 * $SLF_PI)) * 40 ]
    } else {
	return [expr 0]
    }
}

proc BodyTilt {} {
    global WalkTime
    global SLF_PI

    return [expr -7 * cos ( $WalkTime / 20.0 * 2 * $SLF_PI ) ]
}

proc BodyBob {} {
    global WalkTime
    global SLF_PI

    return [expr .01 * cos ( $WalkTime / 20.0 * 4 * $SLF_PI ) ]
}

proc TailTilt {} {
    global WalkTime
    global SLF_PI

    return [expr 2 * cos ( $WalkTime / 20.0 * 4 * $SLF_PI ) ]
}

proc waterCol { offset } {
    global WalkTime
    global SLF_PI

    return [expr .3 * cos ( ($WalkTime + $offset) / 20.0 * 2 * $SLF_PI ) +.7 ]

}



